@@ -1,10 +1,10 @@
 """
-$Id: Base.py,v 1.12.2.10 2008/08/01 03:58:03 customdesigned Exp $
+$Id: Base.py,v 1.12.2.15 2011/03/19 22:15:01 customdesigned Exp $
 
 This file is part of the pydns project.
 Homepage: http://pydns.sourceforge.net
 
-This code is covered by the standard Python License.
+This code is covered by the standard Python License.  See LICENSE for details.
 
     Base functionality. Request and Response classes, that sort of thing.
 """
@@ -31,7 +31,8 @@ class DNSError(Exception): pass
 import Lib
 
 defaults= { 'protocol':'udp', 'port':53, 'opcode':Opcode.QUERY,
-            'qtype':Type.A, 'rd':1, 'timing':1, 'timeout': 30 }
+            'qtype':Type.A, 'rd':1, 'timing':1, 'timeout': 30,
+            'server_rotate': 0 }
 
 defaults['server']=[]
 
@@ -84,6 +85,9 @@ class DnsRequest:
             if len(name) == 1:
                 if name[0]:
                     args['name']=name[0]
+        if defaults['server_rotate'] and \
+                type(defaults['server']) == types.ListType:
+            defaults['server'] = defaults['server'][1:]+defaults['server'][:1]
         for i in defaults.keys():
             if not args.has_key(i):
                 if self.defaults.has_key(i):
@@ -107,21 +111,29 @@ class DnsRequest:
         self.args['server']=self.ns
         return self.processReply()
 
+    def _readall(self,f,count):
+      res = f.read(count)
+      while len(res) < count:
+        if self.timeout > 0:
+            # should we restart timeout everytime we get a dribble of data?
+            rem = self.time_start + self.timeout - time.time()
+            if rem <= 0: raise DNSError,'Timeout'
+            self.s.settimeout(rem)
+        buf = f.read(count - len(res))
+        if not buf:
+          raise DNSError,'incomplete reply - %d of %d read' % (len(res),count)
+        res += buf
+      return res
+
     def processTCPReply(self):
         if self.timeout > 0:
-            r,w,e = select.select([self.s],[],[],self.timeout)
-            if not len(r):
-                raise DNSError, 'Timeout'
+            self.s.settimeout(self.timeout)
+        else:
+            self.s.settimeout(None)
         f = self.s.makefile('r')
-        header = f.read(2)
-        if len(header) < 2:
-            raise DNSError,'EOF'
+        header = self._readall(f,2)
         count = Lib.unpack16bit(header)
-        self.reply = f.read(count)
-        if len(self.reply) != count:
-            # FIXME: Since we are non-blocking, it could just be a large reply
-            # that we need to loop and wait for.
-            raise DNSError,'incomplete reply'
+        self.reply = self._readall(f,count)
         self.time_finish=time.time()
         self.args['server']=self.ns
         return self.processReply()
@@ -272,7 +284,8 @@ class DnsRequest:
                     # FIXME: throws WOULDBLOCK if request too large to fit in
                     # system buffer
                     self.s.sendall(buf)
-                    self.s.shutdown(socket.SHUT_WR)
+                    # SHUT_WR breaks blocking IO with google DNS (8.8.8.8)
+                    #self.s.shutdown(socket.SHUT_WR)
                     r=self.processTCPReply()
                     if r.header['id'] == self.tid:
                         self.response = r
@@ -318,6 +331,21 @@ class DnsAsyncRequest(DnsRequest,asyncore.dispatcher_with_send):
 
 #
 # $Log: Base.py,v $
+# Revision 1.12.2.15  2011/03/19 22:15:01  customdesigned
+# Added rotation of name servers - SF Patch ID: 2795929
+#
+# Revision 1.12.2.14  2011/03/17 03:46:03  customdesigned
+# Simple test for google DNS with tcp
+#
+# Revision 1.12.2.13  2011/03/17 03:08:03  customdesigned
+# Use blocking IO with timeout for TCP replies.
+#
+# Revision 1.12.2.12  2011/03/16 17:50:00  customdesigned
+# Fix non-blocking TCP replies.  (untested)
+#
+# Revision 1.12.2.11  2010/01/02 16:31:23  customdesigned
+# Handle large TCP replies (untested).
+#
 # Revision 1.12.2.10  2008/08/01 03:58:03  customdesigned
 # Don't try to close socket when never opened.
 #
